@@ -13,10 +13,10 @@
 # #'
 # #' This function calculates information entropy (ie) for # multiple x variables.
 # #'
-# #' @param dt A data frame with both x (predictor/feature) and y # (response/label) variables.
+# #' @param dt A data frame with both x (predictor/feature) and # y # (response/label) variables.
 # #' @param y Name of y variable.
-# #' @param x Name of x variables. Default is NULL. If x is NULL, # then all variables except y are counted as x variables.
-# #' @param order Logical, default is TRUE. If it is TRUE, the # output will descending order via ie.
+# #' @param x Name of x variables. Default is NULL. If x is # NULL, # then all variables except y are counted as x # variables.
+# #' @param order Logical, default is TRUE. If it is TRUE, the # # output will descending order via ie.
 # #'
 # #' @return Information Entropy
 # # #' @details
@@ -36,7 +36,7 @@ ie = function(dt, y, x=NULL, order=TRUE) {
   # set dt as data.table
   dt = setDT(dt)
   # remove date/time col
-  dt = rm_datetime_col(dt)
+  dt = rmcol_datetime_unique1(dt)
   # replace "" by NA
   dt = rep_blank_na(dt)
   # check y
@@ -130,10 +130,10 @@ ie_01 = function(good, bad) {
 # #'
 # #' This function calculates gini impurity (used by the CART # Decision Tree) for multiple x variables.
 # #'
-# #' @param dt A data frame with both x (predictor/feature) and y (response/label) variables.
+# #' @param dt A data frame with both x (predictor/feature) and # y (response/label) variables.
 # #' @param y Name of y variable.
-# #' @param x Name of x variables. Default is NULL. If x is NULL, then all variables except y are counted as x variables.
-# #' @param order Logical, default is TRUE. If it is TRUE, the output will descending order via gini
+# #' @param x Name of x variables. Default is NULL. If x is # NULL, then all variables except y are counted as x variables.
+# #' @param order Logical, default is TRUE. If it is TRUE, the # output will descending order via gini
 # #'
 # #' @return gini impurity
 # # #' @details
@@ -147,14 +147,13 @@ ie_01 = function(good, bad) {
 # #'
 # #' @import data.table
 # #' @export
-# #'
 ig = function(dt, y, x=NULL, order=TRUE) {
   gini_impurity = label = NULL
 
   # set dt as data.table
   dt = setDT(dt)
   # remove date/time col
-  dt = rm_datetime_col(dt)
+  dt = rmcol_datetime_unique1(dt)
   # replace "" by NA
   dt = rep_blank_na(dt)
   # check y
